package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.MedCheckInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.MedCheckInfoServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.MedCheckInfoAddVO;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-06 13:27
 */
@Component
public class MedCheckInfoFacade extends MedCheckInfoServiceImpl {

    /**
     * @param medCheckInfoAddVO
     * @return
     */
    public Boolean addMedCheckInfos(MedCheckInfoAddVO medCheckInfoAddVO) {
        boolean res = false;
        Date now = DateUtil.now();
        //获取人员id 和 该人员的医院id
        Long hospitalId = Long.valueOf(SysUserUtils.getCurrentHospitalID());
        String principleName = SysUserUtils.getCurrentPrinciple();
        Long principleId = Long.valueOf(SysUserUtils.getCurrentPrincipleID());
        Map<String, Object> mapAll = new HashMap<String, Object>();
        mapAll.put("is_deleted", IsDeleteEnum.N.getKey());
        mapAll.put("behospital_code", medCheckInfoAddVO.getBehospitalCode());
        mapAll.put("hospital_id", hospitalId);
        QueryWrapper<MedCheckInfo> medCheckInfoQuerys = new QueryWrapper<>();
        medCheckInfoQuerys.allEq(mapAll);
        MedCheckInfo data = this.getOne(medCheckInfoQuerys, false);
        if (data != null) {
            //检验核查人是否是本人
            if (data.getCheckId().equals(principleId)) {
                res = saveNew(mapAll, now, hospitalId, principleId, principleName, medCheckInfoAddVO);
            } else {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据已被人核查，您暂无权限！");
            }
        } else {
            //该病历没有核查的时候， 就添加核查记录
            MedCheckInfo medCheckInfo = new MedCheckInfo();
            medCheckInfo.setGmtCreate(now);
            medCheckInfo.setGmtModified(now);
            medCheckInfo.setHospitalId(hospitalId);
            medCheckInfo.setCheckId(principleId);
            medCheckInfo.setCheckName(principleName);
            medCheckInfo.setCheckTime(now);
            medCheckInfo.setStatus(medCheckInfoAddVO.getCheckStatus());
            medCheckInfo.setBehospitalCode(medCheckInfoAddVO.getBehospitalCode());
            res = this.save(medCheckInfo);
        }

        return res;
    }

    /**
     * @param mapAll
     * @param now
     * @param hospitalId
     * @param principleId
     * @param principleName
     * @param medCheckInfoAddVO
     * @return
     */
    public boolean saveNew(Map<String, Object> mapAll, Date now, Long hospitalId, Long principleId, String principleName, MedCheckInfoAddVO medCheckInfoAddVO) {
        boolean res = false;
        UpdateWrapper<MedCheckInfo> medCheckInfoUpdate = new UpdateWrapper<>();
        medCheckInfoUpdate.allEq(mapAll).set("is_deleted", IsDeleteEnum.Y.getKey()).set("gmt_modified", now);
        res = this.update(medCheckInfoUpdate);
        //删除原来的成功后， 再新建一条核查数据
        if (res) {
            MedCheckInfo medCheckInfo = new MedCheckInfo();
            medCheckInfo.setGmtCreate(now);
            medCheckInfo.setGmtModified(now);
            medCheckInfo.setHospitalId(hospitalId);
            medCheckInfo.setCheckId(principleId);
            medCheckInfo.setCheckName(principleName);
            medCheckInfo.setCheckTime(now);
            medCheckInfo.setStatus(medCheckInfoAddVO.getCheckStatus());
            medCheckInfo.setBehospitalCode(medCheckInfoAddVO.getBehospitalCode());
            res = this.save(medCheckInfo);
        }
        return res;
    }
}
